// Resistor

import java.awt.*;
import java.awt.geom.*;


public class Resistor  {
	String name;
	int n1, n2;
	double value;
	double xpos, ypos;
	double angle, phi;
	Color color;
	AffineTransform tr;

    Resistor()
    {
	color = Color.RED;
	xpos = 0.0;
	ypos = 0.0;
	angle = 0.0;
	
    }

    Resistor(String [] tokens) {
	if (tokens.length>4) {
		name = tokens[1];
		n1 = Integer.parseInt(tokens[2]);
		n2 = Integer.parseInt(tokens[3]);
		value = Double.parseDouble(tokens[4]);
	}
	if (tokens.length>6) {
		xpos = Double.parseDouble(tokens[5]);
		ypos = Double.parseDouble(tokens[6]);
	}
	if (tokens.length>7) setAngle(Double.parseDouble(tokens[7]));
   }

   public String toString() {
	return "R " + name + " " + n1 + " " + n2 + " " + value + " " + xpos + " " + ypos + " " + angle;
   }


    public void setAngle(double angle)
    {
	this.angle = angle;
	phi = angle*Math.PI/180.0;
   }

   public void setPosition(double x, double y) {
	xpos = x;
	ypos = y;
   }

   public boolean getLocation(Point2D src, Point2D dst) {
	tr.transform(src,dst);
	if (Math.abs(dst.getX())>3.1) return false;
	if (Math.abs(dst.getY())>1.3) return false;
	return true;
   }

   public void draw(Graphics g) {
	double scl = 0.1;
	Graphics2D g2 = (Graphics2D) g;
	Stroke saveStroke = g2.getStroke();
	AffineTransform saveXform = g2.getTransform();
	// use minus sign to switch to book/image coordinate system
	g2.translate(xpos,-ypos);
	g2.rotate(-phi);
	g2.scale(scl,scl);
	tr = g2.getTransform();
	try {
	  tr = tr.createInverse();
	}
	catch(Exception e) {
	  System.err.println("Exception " + e);
	}

	g2.setStroke(new BasicStroke(0.25f,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));
	Paint savePaint = g2.getPaint();
	g2.setPaint(color);
	GeneralPath p = new GeneralPath();

	double w = 2.5/7.0;
	double y = 1.25;
	p.moveTo(-3.0,0);
	p.lineTo(-6*w,0);
	p.lineTo(-5*w,-y);
	p.lineTo(-3*w,y);
	p.lineTo(-w,-y);
	p.lineTo(w,y);
	p.lineTo(3*w,-y);
	p.lineTo(5*w,y);
	p.lineTo(6*w,0);
	p.lineTo(3.0,0);
	g2.draw(p);
	g2.setStroke(saveStroke);
	g2.setPaint(savePaint);
	g2.setTransform(saveXform);

/*
	grp.setOnMousePressed( e ->
	{
	    Point2D pt = grp.getParent().sceneToLocal(e.getSceneX(),e.getSceneY());
	    orgSceneX = pt.getX();
	    orgSceneY = pt.getY();
	    orgTranslateX = xpos;
	    orgTranslateY = ypos;
	}
	);

	grp.setOnMouseDragged( e ->
	{
		Point2D pt = grp.getParent().sceneToLocal(e.getSceneX(),e.getSceneY());
		double offsetX = pt.getX() - orgSceneX;
		double offsetY = pt.getY() - orgSceneY;
		setPosition(orgTranslateX+offsetX,orgTranslateY-offsetY);
	}
	);
*/
    }
}
