import java.util.*;
import javax.swing.*;
import Jama.*;

public class Solver {
	ArrayList<Resistor> rlist;
	JTextArea area;

	Solver(JTextArea area) {
		this.area = area;
	}

	public int countNodes() {
		int count = 0;
		for (Resistor r: rlist) {
			if (r.n1>count) count=r.n1;
			if (r.n2>count) count=r.n2;
		}
		return count;
	}



 	public void SolveCircuit() {
        	int numberOfNodes = countNodes();
		if (numberOfNodes==0) return;
       		double a[][] = new double[numberOfNodes][numberOfNodes];
        	double b[][] = new double[numberOfNodes][1];
		// check that new arrays are zero
        	b[0][0] = 1;

        	//Fill matrix a with g values
        	for (Resistor res : rlist) {
	   		double g = 1/res.value;
	   		int i = res.n1 - 1;
	   		int j = res.n2 - 1;
	   		if (i>=0) {
                		a[i][i] += g;
                 		if (j>=0) a[i][j] -= g;
	  		}
	  		if (j>=0) {
                 		a[j][j] += g;
                 		if (i>=0) a[j][i] -= g;
	  		}
        	}
 
 		Matrix A = new Matrix(a);
        	Matrix B = new Matrix(b);

		String fmt = "%8.3f";

		print("\nA =\n");
		print(A,fmt);
		print("b =\n");
		print(B,fmt);

       		Matrix v = A.solve(B);

		print("voltages\n");

		print(v,fmt);
    	}

	private void print(String s) {
		area.append(s);
	}


   	private void print(Matrix M, String fmt) 
	{
     		//String fmt = "%8.4f";
     		int nrows = M.getRowDimension();
     		int ncols = M.getColumnDimension();
     		for (int i = 0;  i<nrows; i++) 
		{
			for (int j=0; j<ncols; j++) 
			{
          			area.append(String.format(fmt,M.get(i,j)));
        		}
        		area.append("\n");
     		}
		area.append("\n");
   	}
}