// Testbed3.java
import java.awt.*;
import java.util.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.*;

public class Testbed3 extends JPanel implements MouseMotionListener
{
    final static int  resolution = 96;
    AffineTransform tf;
    ArrayList<Resistor> rlist = new ArrayList<Resistor>();
    CurrentSource i1 = new CurrentSource();
    JLabel status = new JLabel(" ");

    Testbed3()
    {
	Resistor r1 = new Resistor();
	r1.name = "R1";
    	rlist.add(r1);
        Resistor r2 = new Resistor();
        r2.setPosition(0.0,1.0);
        r2.setAngle(30.0);
        r2.color = Color.BLUE;
	r2.name = "R2";
	rlist.add(r2);

        i1.setPosition(-1.0,0.0);

	status.setBackground(Color.WHITE);
	status.setOpaque(true);
	status.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));

     	addMouseMotionListener( this );
    }


    protected void show_size(Graphics g)
    {
	int width = getWidth();
	int height = getHeight();
	String str = String.format("width %d height %d",width,height);
	FontMetrics fm = g.getFontMetrics();
	Rectangle2D rect = fm.getStringBounds(str,g);
	width -= rect.getWidth();
	height -= rect.getHeight();
	g.drawString(str,width/2,height/2);
    }
    protected void draw_grid(Graphics g)
    {
	Graphics2D g2 = (Graphics2D) g;
	AffineTransform saveXform = g2.getTransform();
	Paint savePaint = g2.getPaint();
	g2.setPaint(new Color(220,220,255));
	Stroke saveStroke = g2.getStroke();
	g2.setStroke(new BasicStroke(1.0f/resolution));
	double u = 4.0;
	for (int i=-4; i<=4; i++) {
		double v = i;			
		g2.draw(new Line2D.Double(-u,v,4,v));
		g2.draw(new Line2D.Double(v,-u,v,u));
	}
	// locate origin
	boolean show_origin = false;
	if (show_origin) {
  	   g2.setPaint(Color.BLACK);
	   u = 0.05;
	   g2.draw(new Line2D.Double(-u,u,u,-u));
	   g2.draw(new Line2D.Double(-u,-u,u,u));
	}
	g2.setStroke(saveStroke);
	g2.setTransform(saveXform);
    }

    public void paintComponent(Graphics g)
    {
	super.paintComponent( g );
	Graphics2D g2 = (Graphics2D) g;
	AffineTransform saveXform = g2.getTransform();
	Paint savePaint = g2.getPaint();
	int width = getWidth();
	int height = getHeight();
	g2.translate(width/2,height/2);
	g2.scale(resolution,resolution);
	try {
	  tf = g2.getTransform().createInverse();
	}
	catch(Exception e) {
	  System.err.println("Exception " + e);
	}

	draw_grid(g);
	Stroke saveStroke = g2.getStroke();
	g2.setStroke(new BasicStroke(2.0f/resolution,BasicStroke.CAP_ROUND,BasicStroke.JOIN_ROUND));

	i1.draw(g);
	for (Resistor r: rlist) r.draw(g);

	g2.setTransform(saveXform);
	g2.setPaint(savePaint);
	g2.setStroke(saveStroke);
	// show_size(g);
    }

    public void mouseDragged( MouseEvent e )
   {
      status.setText( "Dragged at [" + e.getX() +
                         ", " + e.getY() + "]" );
   }

   public void mouseMoved( MouseEvent e )
   {
	Point2D src = new Point2D.Double(e.getX(),e.getY());
	Point2D dst = new Point2D.Double();
	tf.transform(src,dst);
	Point2D dst1 = new Point2D.Double();
	boolean inside = false;
	String str = String.format("Moved at [ %.2g, %.2g]",dst.getX(),-dst.getY());
	for (Resistor r: rlist) {
		inside = r.getLocation(src,dst1); 
		if (inside) {
			str = String.format("inside %s: [%.2g %.2g]",r.name, dst1.getX(),-dst1.getY());
			break;
		}
	}
        status.setText( str );
   }

    public static void main(String[] args)
    {
        EventQueue.invokeLater(() -> {
		Testbed3 panel = new Testbed3();
		JFrame application = new JFrame("Testbed3");
		application.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		application.add(panel);
		application.add(panel.status,BorderLayout.SOUTH);
		application.setSize(496,400);
		application.setVisible(true);
	});
    }
}